"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mysql_1 = require("mysql");
const builder_1 = require("./builder");
const trigger_promise_1 = require("./trigger_promise");
function select(columns, exec) {
    const sql = {
        select: 'SELECT',
    };
    if (columns instanceof Array) {
        sql.columns = columns.map((c) => mysql_1.escapeId(c)).join(", ");
    }
    else {
        sql.columns = columns;
    }
    return new trigger_promise_1.default((resolve, reject) => {
        exec(builder_1.joinSql([sql.select, sql.columns, sql.from, sql.where, sql.groupBy,
            sql.orderBy, sql.limit, sql.offset]), resolve, reject);
    }, {
        from(table) {
            sql.from = `FROM ${mysql_1.escapeId(table)}`;
            return this;
        },
        where(...args) {
            sql.where = builder_1.buildWhere(...args);
            return this;
        },
        groupBy(column) {
            sql.groupBy = builder_1.buildGroupBy(column);
            return this;
        },
        orderBy(...orders) {
            sql.orderBy = builder_1.buildOrderBy(...orders);
            return this;
        },
        limit(num) {
            sql.limit = builder_1.buildLimit(num);
            return this;
        },
        offset(num) {
            sql.offset = builder_1.buildOffset(num);
            return this;
        },
    });
}
exports.select = select;
function count(table, exec) {
    const sql = {
        select: `SELECT COUNT(*) AS count FROM ${mysql_1.escapeId(table)}`,
        where: "",
    };
    return new trigger_promise_1.default((resolve, reject) => {
        exec(builder_1.joinSql([sql.select, sql.where]), resolve, reject);
    }, {
        where(...args) {
            sql.where = builder_1.buildWhere(...args);
            return this;
        },
    });
}
exports.count = count;
function insertInto(table, exec) {
    const sql = {
        insert: `INSERT INTO`,
        table: "",
        values: "",
    };
    return new trigger_promise_1.default((resolve, reject) => {
        exec(builder_1.joinSql([sql.insert, sql.table, sql.values]), resolve, reject);
    }, {
        values(values) {
            const keys = [];
            values.forEach((value) => Object.keys(value).forEach((k) => {
                if (keys.indexOf(k) === -1) {
                    keys.push(k);
                }
            }));
            sql.table = `${mysql_1.escapeId(table)} (${keys.map((k) => mysql_1.escapeId(k)).join(",")}) VALUES`;
            sql.values = values.map((value) => {
                const arr = new Array(keys.length);
                Object.keys(value).forEach((k) => {
                    arr[keys.indexOf(k)] = value[k];
                });
                return `(${mysql_1.escape(arr)})`;
            }).join(",");
            return this;
        },
    });
}
exports.insertInto = insertInto;
function deleteFrom(table, exec) {
    const sql = {
        deleteFrom: `DELETE FROM ${mysql_1.escapeId(table)}`,
    };
    return new trigger_promise_1.default((resolve, reject) => {
        exec(builder_1.joinSql([sql.deleteFrom, sql.where, sql.limit, sql.offset]), resolve, reject);
    }, {
        where(...args) {
            sql.where = builder_1.buildWhere(...args);
            return this;
        },
        limit(num) {
            sql.limit = builder_1.buildLimit(num);
            return this;
        },
        offset(num) {
            sql.offset = builder_1.buildOffset(num);
            return this;
        },
    });
}
exports.deleteFrom = deleteFrom;
function update(table, exec) {
    const sql = {
        update: `UPDATE ${mysql_1.escapeId(table)}`,
    };
    return new trigger_promise_1.default((resolve, reject) => {
        exec(builder_1.joinSql([sql.update, sql.set, sql.where]), resolve, reject);
    }, {
        where(...args) {
            sql.where = builder_1.buildWhere(...args);
            return this;
        },
        set(value) {
            sql.set = `SET ${mysql_1.escape(value)}`;
            return this;
        },
    });
}
exports.update = update;
//# sourceMappingURL=query.js.map